/*
 * Copyright (c) 2004, Stefan Walter
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Stef Walter <stef@memberwebs.com>
 *
 */ 

#include "usuals.h"
#include "levelhandler.h"

/* ----------------------------------------------------------------------------------
 *  CONSTRUCTION
 */
LevelHandler::LevelHandler()
{
    m_parser = NULL;
}

LevelHandler::~LevelHandler()
{
	clear();
}

void LevelHandler::clear()
{
	m_curLevel.release();
	m_topLevel.release();

    m_parser = NULL;
}

/* ----------------------------------------------------------------------------------
 *  OVERRIDES
 */
 
void LevelHandler::startDocument(RtfParser* parser)
{
    clear();
    
    m_parser = parser;
    
	m_topLevel = new Level;
	m_curLevel = m_topLevel;
}

void LevelHandler::endDocument()
{

}

void LevelHandler::groupStart()
{
	ASSERT(m_curLevel);
	pushLevel();
}

void LevelHandler::groupEnd()
{
	ASSERT(m_curLevel);
	popLevel();
}

/* ----------------------------------------------------------------------------------
 *  OPERATIONS
 */

DOM::Element LevelHandler::getElement()
{
	ASSERT(m_curLevel);
	return m_curLevel->getElement();
}

void LevelHandler::pushLevel()
{
	// Push a level on the stack
	m_curLevel = m_curLevel->pushLevel();
}

void LevelHandler::popLevel()
{
	// Pull a level off the stack
	LevelPtr level = m_curLevel->getPrevious();

	if(level)
		m_curLevel = level;
}

void LevelHandler::rewindLevel(LevelPtr ptr)
{
	ASSERT(ptr != NULL);

	LevelPtr prev = ptr->getPrevious();

	if(prev != NULL)
		m_curLevel = prev;
	else
		m_curLevel = ptr;
}

LevelPtr LevelHandler::getLevel()
{
	return m_curLevel;
}
