/*
 * Copyright (c) 2004, Stefan Walter
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Stef Walter <stef@memberwebs.com>
 *
 */ 
 
#ifndef __LEVELHANDLER_H__
#define __LEVELHANDLER_H__

#include "rtfparser.h"
#include "reference.h"
#include "xmlcomposehelpers.h"

/*
 * LevelHandler
 * 
 * A base class that manages a set of Levels (see xmlcomposehelpers.cpp)
 * based on the RTF groups seen.
 */
class LevelHandler 
	: public RtfHandler  
{
public:
	LevelHandler();
	virtual ~LevelHandler();

    // Overrides
	virtual void startDocument(RtfParser* parser);
	virtual void endDocument();
	virtual void groupStart(); 
	virtual void groupEnd();
   
    // Convenience function to get XML element from current level
	virtual DOM::Element getElement();

    // Create a new level on top of stack
	void pushLevel();
 
    // Pop top level and discard
	void popLevel();
  
    // Back out all the way past a given level
	void rewindLevel(LevelPtr ptr);
   
    // Get the current level
	LevelPtr getLevel();

protected:
	virtual void clear();

	LevelPtr m_topLevel;       // First level 
	LevelPtr m_curLevel;       // The current level
    RtfParser* m_parser;       // The parser we're listening to
};

#endif // __LEVELHANDLER_H__
