/*
 * Copyright (c) 2004, Stef Walter
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Stef Walter <stef@memberwebs.com>
 *
 */ 
 
#ifndef __TAGS_H__
#define __TAGS_H__

/*
 * IMPORTANT: When adding tags, attributes or values here make sure to update 
 * the XSL Schema for the output formats. 
 */

static const char* kNSPrefix = "xmlns";
static const char* kNSRtfx = "http://memberwebs.com/ns/rtfx/";

// Tags used internally
static const char* kElDest = "i_dest";
static const char* kElBlock = "i_block";
static const char* kElListtable = "i_listtable";
static const char* kElListdef = "i_listdef";
static const char* kElFontTable = "i_fonttable";
static const char* kElFontDef = "i_fontdef";
static const char* kAtFix = "i_fix";
static const char* kAtCell = "i_cell";
static const char* kAtList = "i_list";

// All the main tags
static const char* kElPara = "para";
static const char* kElDoc = "document";
static const char* kElTab = "tab";
static const char* kElSect = "sect";
static const char* kElPage = "page";
static const char* kElStyle = "style";
static const char* kElSpan = "span";
static const char* kElLine = "line";
static const char* kElList = "list";
static const char* kElStylesheet = "stylesheet";
static const char* kElInfo = "info";
static const char* kElTitle = "title";
static const char* kElAuthor = "author";
static const char* kElOperator = "operator";
static const char* kElB = "b";
static const char* kElHide = "hide";
static const char* kElI = "i";
static const char* kElStrike = "strike";
static const char* kElU = "u";
static const char* kElSuper = "super";
static const char* kElSub = "sub";
static const char* kElHighlight = "highlight";
static const char* kElCell = "cell";
static const char* kElRow = "row";
static const char* kElTable = "table";
static const char* kElFootNote = "footnote";
static const char* kElBookmark = "bookmark";
static const char* kElRef = "ref";
static const char* kElFont = "font";
static const char* kElOptions = "options";

// Attributes
static const char* kAtName = "name";
static const char* kAtBold = "bold";
static const char* kAtHidden = "hide";
static const char* kAtItalic = "italic";
static const char* kAtStrike = "strike";
static const char* kAtUnderline = "underline";
static const char* kAtColor = "color";
static const char* kAtType = "type";
static const char* kAtOrdered = "ordered";
static const char* kAtStart = "start";
static const char* kAtId = "id";
static const char* kAtTo = "to";
static const char* kAtSize = "size";
static const char* kAtAlign = "align";
static const char* kAtIndent = "indent";

// Values
static const char* kValDisc = "disc";
static const char* kValLowerAlpha = "lower-alpha";
static const char* kValUpperAlpha = "upper-alpha";
static const char* kValLowerRoman = "lower-roman";
static const char* kValUpperRoman = "upper-roman";
static const char* kValArabic = "arabic";
static const char* kValNull = "";
static const char* kValFootNote = "footnote";
static const char* kValBookmark = "bookmark";
static const char* kValHyperlink = "hyperlink";
static const char* kValList = "list";
static const char* kValPara = "para";
static const char* kValTable = "table";
static const char* kValTrue = "true";
static const char* kValZero = "0";
static const char* kValCenter = "center";
static const char* kValRight = "right";
static const char* kValJustify = "justify";

#endif // __TAGS_H__
