/*
 * Copyright (c) 2004, Stefan Walter
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Stef Walter <stef@memberwebs.com>
 *
 */ 

#include "usuals.h"
#include "xmlcomposehelpers.h"

/* ----------------------------------------------------------------------------------
 *  CONSTRUCTION
 */
 
Level::Level()
{
	m_previous = NULL;
	m_element = NULL;
	m_destination = NULL;
	m_analyser = NULL;
}

Level::Level(const Level& level) :
	m_text(level.m_text)
{
	m_element = NULL;
	m_destination = NULL;
	m_analyser = NULL;
	m_previous = &level;
}

Level::~Level()
{
	if(m_analyser)
		m_analyser->done();
	if(m_destination)
		m_destination->done();
}

/* ----------------------------------------------------------------------------------
 *  LEVEL OPERATIONS 
 */

LevelPtr Level::pushLevel()
{
	LevelPtr level = new Level(*this);
	return level;
}

LevelPtr Level::getPrevious()
{
	return m_previous; 
}

DOM::Element Level::getElement(bool deep)
{
	if(m_element != NULL)
		return m_element;
	else if(deep && m_previous)
		return m_previous->getElement();
	else
    	return DOM::Element();
}

void Level::setElement(DOM::Element element, bool deep)
{
	if(deep && m_previous && m_element == NULL)
		m_previous->setElement(element, deep);
	else
		m_element = element;
}

AnalyserPtr Level::getAnalyser(bool deep)
{
	if(m_analyser)
		return m_analyser;
	else if(deep && m_previous)
		return m_previous->getAnalyser();
	else
		return NULL;
}

void Level::setAnalyser(AnalyserPtr analyser, bool deep)
{
	if(deep && m_previous && !m_analyser)
		m_previous->setAnalyser(analyser, deep);

	else
	{
		if(m_analyser != NULL)
			m_analyser->done();

		m_analyser = analyser;
	}
}

DestinationPtr Level::getDestination(bool deep)
{
	if(m_destination)
		return m_destination;
	else if(deep && m_previous)
		return m_previous->getDestination();
	else
		return NULL;
}

void Level::setDestination(DestinationPtr destination, bool deep)
{
	if(deep && m_previous && !m_destination)
		m_previous->setDestination(destination, deep);
	else
	{
		if(m_destination != NULL)
			m_destination->done();

		m_destination = destination;
	}
}

RtfFormatting& Level::getFormatting()
{
	return m_text;
}

void Level::setTextProperties(RtfFormatting& formatting)
{
	m_text.copy(formatting);
}
